"""
Module containing functions that auto-generate a schedule.ccl file for an input thorn.

Author: Zachariah B. Etienne
        zachetie **at** gmail **dot* com
        Steven R. Brandt
        sbrandt **at** cct **dot** lsu **dot** edu
"""

from pathlib import Path

from nrpy.helpers.conditional_file_updater import ConditionalFileUpdater


def construct_configuration_ccl(
    project_dir: str,
    thorn_name: str,
) -> None:
    """
    Construct the ConfigurationCCL string based on its properties.

    :param project_dir: The directory of the project.
    :param thorn_name: The name of the thorn.
    """
    outstr = """# This configuration.ccl file was automatically generated by NRPy+.
#   You are advised against modifying it directly; instead
#   modify the Python code that generates it.
REQUIRES Loop CarpetX
"""
    output_Path = Path(project_dir) / thorn_name
    output_Path.mkdir(parents=True, exist_ok=True)
    with ConditionalFileUpdater(
        output_Path / "configuration.ccl", encoding="utf-8"
    ) as fd:
        fd.write(outstr)
